#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <linux/fb.h>
#include <sys/ioctl.h>
#include <sys/kd.h>


#define IMAGE_INTERVAL  10
#define BITS_PER_BYTE   8

void show_bitmap (const char *image_name)
{
    int status, image_fd, fb_fd;
    struct fb_var_screeninfo fb_vs;
    char *buffer = NULL;
    unsigned int screen_size = 0;

    fb_fd = open("/dev/fb0", O_RDWR);
    if (!fb_fd) {
        fprintf(stderr,"Could not open framebuffer.\n");
        exit(1);
    }

    if (ioctl (fb_fd, FBIOGET_VSCREENINFO, &fb_vs) == -1)
    {
        fprintf(stderr,"Failed to get screen info from the framebuffer\n");
        exit(1);
    }
    screen_size = (fb_vs.xres * fb_vs.yres * ( fb_vs.bits_per_pixel / BITS_PER_BYTE ));
    buffer = malloc(sizeof *buffer * screen_size);
    if (buffer == NULL) {
        fprintf(stderr,"Failed to allocate buffer of size: %d\n", screen_size);
        exit(1);
    }

    image_fd = open(image_name, O_RDONLY);
    if (image_fd < 0)
    {
        fprintf(stderr,"Failed to open image: %s\n", image_name);
        exit(1);
    }

    status = read(image_fd, (char*)buffer, screen_size);
    if (status < 0)
    {
        fprintf(stderr,"Failed to read image: %s\n", image_name);
        exit(1);
    }
    status = write(fb_fd, (const char*)buffer, screen_size);
    if (status < 0)
    {
        fprintf(stderr,"Failed to write image to buffer.\n");
        exit(1);
    }
    close(image_fd);
    close(fb_fd);
    free(buffer);
}

int main (int argc, char *argv[])
{
    int index = 1;
    int vconsole_fd; 

    if (argc < 2)
    {
        printf("Usage: %s [image] [image...]\n", argv[0]);
        exit(1);
    }
    
    /* open the framebuffer virtual console */
    vconsole_fd = open("/dev/tty0", O_RDWR);
    if (!vconsole_fd) {
        fprintf(stderr,"Could not open virtual console.\n");
        exit(1);
    }
    
    /* disable blanking on the console by setting the KD_GRAPHICS mode */
    if (ioctl( vconsole_fd, KDSETMODE, KD_GRAPHICS))
    {
        fprintf(stderr,"Could not set virtual console to KD_GRAPHICS mode.\n");
        exit(1);      
    }

    close(vconsole_fd);

    /* no chdir and no stdio close */
    daemon(1,1);

    while (1)
    {
        show_bitmap(argv[index]);

        index++;
        if (index >= argc)
            index = 1;

        sleep(IMAGE_INTERVAL);
    }
}
